/*
 * Copyright © 2020-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of gaming-core.
 *
 * Gaming-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Gaming-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * gaming-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.core.domain;

import java.util.List;

import de.fhdw.gaming.core.ui.InputProvider;
import de.fhdw.gaming.core.ui.InputProviderException;

/**
 * Allows to create a {@link GameBuilder}.
 */
public interface GameBuilderFactory {

    /**
     * Parameter for the name of a player.
     */
    String PARAM_PLAYER_NAME = "playerName";
    /**
     * Parameter for a player's strategy.
     */
    String PARAM_PLAYER_STRATEGY = "playerStrategy";
    /**
     * Parameter for the maximum computation time per move in seconds.
     */
    String PARAM_MAX_COMPUTATION_TIME_PER_MOVE = "maxComputationTimePerMove";

    /**
     * Returns the name of the game.
     */
    String getName();

    /**
     * Returns the minimum number of players in a game.
     */
    int getMinimumNumberOfPlayers();

    /**
     * Returns the maximum number of players in a game.
     */
    int getMaximumNumberOfPlayers();

    /**
     * Returns all strategies.
     */
    List<? extends Strategy<?, ?, ?>> getStrategies();

    /**
     * Creates a {@link GameBuilder} object.
     *
     * @param inputProvider The input provider to use (if necessary) in order to determine mandatory or optional
     *                      parameters for creating the game. Mandatory parameters are the players' names and
     *                      strategies. All other parameters need to have meaningful default values.
     *                      <p>
     *                      The order of the data sets request is as follows:
     *                      <ol>
     *                      <li>Parameters for the game as a whole</li>
     *                      <li>Parameters for player 1</li>
     *                      <li>Parameters for player 2</li>
     *                      <li>Parameters for player ...</li>
     *                      </ol>
     * @return A {@link GameBuilder} which allows to create a game as many times as desired.
     * @throws GameException if the game creation fails for some reason.
     */
    GameBuilder createGameBuilder(InputProvider inputProvider) throws GameException;

    /**
     * Returns the decorated {@link InputProvider}.
     *
     * @param inputProvider The input provider to use (if necessary) in order to
     *                      determine mandatory or optional parameters for creating
     *                      the game. Mandatory parameters are the players' names
     *                      and strategies. All other parameters need to have
     *                      meaningful default values.
     *                      <p>
     *                      The order of the data sets request is as follows:
     *                      <ol>
     *                      <li>Parameters for the game as a whole</li>
     *                      <li>Parameters for player 1</li>
     *                      <li>Parameters for player 2</li>
     *                      <li>Parameters for player ...</li>
     *                      </ol>
     * @return An extended {@link InputProvider}.
     * @throws InputProviderException if the decoration fails for some reason.
     */
    default InputProvider extendInputProvider(final InputProvider inputProvider) throws InputProviderException {
        return inputProvider;
    }
}
