/*
 * Copyright © 2020-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of gaming-core.
 *
 * Gaming-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Gaming-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * gaming-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.core.domain;

/**
 * Represents a move.
 * <p>
 * Note that a move is not a subtype of {@link Stateful}, i.e. it is assumed that a move has no mutable state.
 *
 * @param <P> The type of allowed players.
 * @param <S> The type of allowed states.
 */
public interface Move<P extends Player<P>, S extends State<P, S>> {

    /**
     * Applies this move to a state for some player.
     *
     * @param state  The state to apply this move to.
     * @param player The player making the move.
     * @throws GameException if the rules of the game do not allow this move at this state. Note that this operation
     *                       only performs local state changes. Global state changes are checked in
     *                       {@link State#nextTurn()}. This allows a game move to consist of a sequence of smaller
     *                       (sub-)moves where global consistency is reached only after the last (sub-)move. Of course,
     *                       only the rules of the game eventually dictate which rule can be checked at which point.
     */
    void applyTo(S state, P player) throws GameException;
}
