/*
 * Copyright © 2020-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of gaming-core.
 *
 * Gaming-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Gaming-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * gaming-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.core.domain;

import java.util.Map;
import java.util.Set;

/**
 * Represents the state of a game.
 * <p>
 * Note that CRTP (Curiously Recurring Template Pattern) is used to retain the
 * subtype of the state. This allows clients to clone a state via
 * {@link #deepCopy()} while retaining its type at compile-time.
 *
 * @param <P> The type of allowed players.
 * @param <S> The type of game states.
 */
public interface State<P extends Player<P>, S extends State<P, S>> extends Stateful {

    /**
     * Returns all the players of the game.
     */
    Map<String, P> getPlayers();

    /**
     * Computes the set of players being able to do the next move.
     *
     * @return A set of players that can act next (possibly in parallel). Can be
     *         empty if no players have remained.
     */
    Set<P> computeNextPlayers();

    /**
     * This operation is called after each move and also if no valid move has been
     * found for the set of currently active players.
     */
    void nextTurn();

    @Override
    S deepCopy();
}
