/*
 * Copyright © 2020-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of gaming-core.
 *
 * Gaming-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Gaming-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * gaming-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.core.ui.type;

import java.util.Collections;
import java.util.Optional;
import java.util.Set;

import de.fhdw.gaming.core.ui.InputProviderException;

/**
 * Represents the type of an object field.
 */
public final class ObjectFieldType implements FieldType<Object> {

    /**
     * The default value.
     */
    private final Optional<Object> defaultValue;
    /**
     * The set of allowed values.
     */
    private final Set<?> allowedValues;

    /**
     * Creates a {@link ObjectFieldType}.
     *
     * @param defaultValue  The default value.
     * @param allowedValues The set of allowed values.
     */
    public ObjectFieldType(final Optional<Object> defaultValue, final Set<?> allowedValues) {
        this.defaultValue = defaultValue;
        this.allowedValues = allowedValues;
    }

    /**
     * Returns the allowed values.
     */
    public Set<?> getAllowedValues() {
        return Collections.unmodifiableSet(this.allowedValues);
    }

    @Override
    public void accept(final FieldTypeVisitor visitor) throws FieldTypeException {
        visitor.handle(this);
    }

    @Override
    public Optional<Object> getDefaultValue() {
        return this.defaultValue;
    }

    @Override
    public Optional<String> getInfo() {
        return Optional.of(String.format("The value must be one of %s.", this.allowedValues));
    }

    @Override
    public void preInput(final String id) throws InputProviderException {
        if (this.allowedValues.isEmpty()) {
            throw new InputProviderException(String.format("Field %s: No allowed values.", id));
        }
    }

    @Override
    public Object postInput(final String id, final String value) throws InputProviderException {
        final Optional<?> object = this.allowedValues.stream()
                .filter((final Object o) -> o.toString().equals(value)).findFirst();

        if (object.isEmpty()) {
            throw new InputProviderException(this.getInfo().orElseThrow());
        }
        return object.get();
    }
}
