/*
 * Copyright © 2020-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of gaming-core.
 *
 * Gaming-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Gaming-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * gaming-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.core.ui.util;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Optional;
import java.util.Set;

import de.fhdw.gaming.core.domain.GameBuilder;
import de.fhdw.gaming.core.ui.InputProvider;
import de.fhdw.gaming.core.ui.type.BooleanFieldType;
import de.fhdw.gaming.core.ui.type.FieldType;
import de.fhdw.gaming.core.ui.type.IntegerFieldType;
import de.fhdw.gaming.core.ui.type.ObjectFieldType;
import de.fhdw.gaming.core.ui.type.StringFieldType;
import de.fhdw.gaming.core.ui.type.validator.Validator;

/**
 * A non-interactive {@link InputProvider} implementation returning predefined data only.
 * <p>
 * Unlike interactive input providers, it is possible to create chains of {@link NonInteractiveInputProvider}s in
 * advance. This is helpful e.g. when creating a {@link GameBuilder}, as building a game requires parameters for the
 * game as a whole as well as parameters for each participating player. Each parameter set is expected to be returned by
 * a separate {@link InputProvider}.
 */
public final class NonInteractiveInputProvider implements InputProvider {

    /**
     * The provided data.
     */
    private final Map<String, Object> providedData;
    /**
     * The needed data.
     */
    private final Map<String, FieldType<?>> neededData;

    /**
     * Creates a {@link NonInteractiveInputProvider}.
     */
    public NonInteractiveInputProvider() {
        this.providedData = new LinkedHashMap<>();
        this.neededData = new LinkedHashMap<>();
    }

    @Override
    @SafeVarargs
    public final NonInteractiveInputProvider needString(final String id, final String prompt,
            final Optional<String> defaultValue, final Validator<String>... validators) {

        this.neededData.put(id, new StringFieldType(defaultValue).validateBy(validators));
        return this;
    }

    @Override
    public NonInteractiveInputProvider fixedString(final String id, final String fixedValue) {
        this.providedData.put(id, fixedValue);
        return this;
    }

    @Override
    @SafeVarargs
    public final NonInteractiveInputProvider needInteger(final String id, final String prompt,
            final Optional<Integer> defaultValue, final Validator<Integer>... validators) {

        this.neededData.put(id, new IntegerFieldType(defaultValue).validateBy(validators));
        return this;
    }

    @Override
    public NonInteractiveInputProvider fixedInteger(final String id, final Integer fixedValue) {
        this.providedData.put(id, fixedValue);
        return this;
    }

    @Override
    @SafeVarargs
    public final NonInteractiveInputProvider needBoolean(final String id, final String prompt,
            final Optional<Boolean> defaultValue, final Validator<Boolean>... validators) {

        this.neededData.put(id, new BooleanFieldType(defaultValue).validateBy(validators));
        return this;
    }

    @Override
    public NonInteractiveInputProvider fixedBoolean(final String id, final Boolean fixedValue) {
        this.providedData.put(id, fixedValue);
        return this;
    }

    @Override
    public NonInteractiveInputProvider needObject(final String id, final String prompt,
            final Optional<Object> defaultValue,
            final Set<?> objectSet) {

        this.neededData.put(id, new ObjectFieldType(defaultValue, objectSet));
        return this;
    }

    @Override
    public NonInteractiveInputProvider fixedObject(final String id, final Object fixedValue) {
        this.providedData.put(id, fixedValue);
        return this;
    }

    /**
     * {@inheritDoc}
     * <p>
     * Note that if no predefined value is available for a given field, {@code null} is stored.
     */
    @Override
    public Map<String, Object> requestData(final String title) {
        final Map<String, Object> result = new LinkedHashMap<>();
        for (final Map.Entry<String, FieldType<?>> entry : this.neededData.entrySet()) {
            final FieldType<?> fieldType = entry.getValue();
            result.put(
                    entry.getKey(),
                    this.providedData.getOrDefault(entry.getKey(), fieldType.getDefaultValue().orElse(null)));
        }

        return result;
    }

    @Override
    public NonInteractiveInputProvider getNext(final Map<String, Object> lastDataSet) {
        return new NonInteractiveInputProvider();
    }
}
